@@ -577,6 +577,7 @@ static int detailed_profiles(FILE *outf, filters_t *filters, bool json,
 			     struct profile *profiles, size_t nprofiles) {
 	int ret;
 	size_t i;
+	int is_first = 1;
 
 	if (json) {
 		fprintf(outf, "\"profiles\": {");
@@ -614,7 +615,8 @@ static int detailed_profiles(FILE *outf, filters_t *filters, bool json,
 		for (j = 0; j < nfiltered; j++) {
 			if (json) {
 				fprintf(outf, "%s\"%s\": \"%s\"",
-				       i == 0 && j == 0 ? "" : ", ", filtered[j].name, profile_statuses[i]);
+				       is_first ? "" : ", ", filtered[j].name, profile_statuses[i]);
+				is_first = 0;
 			} else {
 				dfprintf(outf, "   %s\n", filtered[j].name);
 			}
@@ -1036,7 +1038,7 @@ int main(int argc, char **argv)
 	 * have policy associated.
 	 */
 	ret = get_profiles(fp, &profiles, &nprofiles);
-	if (ret != 0) {
+	if (ret != 0 && !opt_json) {
 		eprintf("Failed to get profiles: %d....\n", ret);
 		goto out;
 	}
