@@ -21,7 +21,7 @@ pwd=`cd $pwd ; /bin/pwd`
 
 bin=$pwd
 
-. $bin/prologue.inc
+. "$bin/prologue.inc"
 
 # Read permission was required for a confined process to be able to be traced 
 # using ptrace.  This stopped being required or functioning correctly 
@@ -31,7 +31,7 @@ bin=$pwd
 helper=$pwd/ptrace_helper
 
 bin_true=${tmpdir}/true
-cp -pL /bin/true ${tmpdir}/true
+cp -pL ./true ${tmpdir}/true
 
 # -n number of syscalls to perform
 # -c have the child call ptrace_me, else parent does ptrace_attach
@@ -55,8 +55,8 @@ runchecktest "test 2 -h prog" pass -h -n 100 $helper ${bin_true}
 runchecktest "test 2 -hc prog" pass -h -c -n 100 $helper ${bin_true}
 
 
-if [ "$(kernel_features ptrace)" == "true" -a "$(parser_supports 'ptrace,')" == "true" ] ; then
-	. $bin/ptrace_v6.inc
+if [ "$(kernel_features ptrace)" = "true" -a "$(parser_supports 'ptrace,')" = "true" ] ; then
+	. "$bin/ptrace_v6.inc"
 else
-	. $bin/ptrace_v5.inc
+	. "$bin/ptrace_v5.inc"
 fi
